﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Data.Services.Client;
using System.Linq;
using System.Collections.ObjectModel;
using WP7App.CustomerOrderService;
using WP7App.ViewModels;


namespace WP7App
{
  public class DataAccess : INotifyPropertyChanged
  {
    public ObservableCollection<LineItemViewModel> LineItems { get; private set; }
    public ObservableCollection<OrderViewModel> Orders { get; private set; }
    public CustomerViewModel Customer { get; private set; }
    private CustomerOrderContext _context;
    private Customer _currentCustomerEntity;

    public delegate void SaveStatusChangedEventHandler(object sender, PropertyChangedEventArgs e );


    public DataAccess()
    {
      _context = new CustomerOrderContext(new Uri("http://localhost:18049/CustomerDataService.svc/"));
      LineItems = new ObservableCollection<LineItemViewModel>();
      Orders = new ObservableCollection<OrderViewModel>();
      Customer = new CustomerViewModel();
    }

    public string CurrentOrderNumber { get; private set; }
    public bool IsDataLoaded{get;private set;}

    public void LoadData()
    {
      GetCustomerAndOrders();
    }

    private void GetCustomerAndOrders()
    {
      var qry = (DataServiceQuery<Customer>)(from c in _context.Customers.Expand("Orders") select c).Take(1);

      try
      {
        qry.BeginExecute(CustomersAndOrdersRetrieved, qry);
      }
      catch (DataServiceQueryException ex)
      {
        throw new ApplicationException(
            "An error occurred during query execution.", ex);
      }

    }

    private void CustomersAndOrdersRetrieved(IAsyncResult result)
    {
      try
      {
        var query =
            result.AsyncState as DataServiceQuery<Customer>;

        _currentCustomerEntity = query.EndExecute(result).FirstOrDefault() as Customer;


        Customer = new CustomerViewModel
        {
          CustomerId = _currentCustomerEntity.CustomerId,
          Email = _currentCustomerEntity.EmailAddress,
          FirstName = _currentCustomerEntity.FirstName,
          LastName = _currentCustomerEntity.LastName,
          Phone = _currentCustomerEntity.Phone
        };

        var orders = new List<OrderViewModel>();
        foreach (var order in _currentCustomerEntity.Orders)
        {
          orders.Add(new OrderViewModel
                            {
                              OrderId = order.OrderId,
                              OrderDate = order.OrderDate,
                              OrderNumber = order.SalesOrderNumber,
                              SubTotal = order.SubTotal
                            });
        }

        this.Orders = new ObservableCollection<OrderViewModel>(orders);

        NotifyPropertyChanged("Customer");
      }
      catch (Exception ex)
      {

        NotifyPropertyChanged("Error:" + ex.Message);
      }


      this.IsDataLoaded = true;
    }

    public void Save()
    {

      _currentCustomerEntity.EmailAddress = Customer.Email;
      _currentCustomerEntity.FirstName = Customer.FirstName;
      _currentCustomerEntity.LastName = Customer.LastName;
      _currentCustomerEntity.Phone = Customer.Phone;

      _context.UpdateObject(_currentCustomerEntity);
      _context.BeginSaveChanges(FinishSaveChanges, _context);

    }

    private void FinishSaveChanges(IAsyncResult result)
    {
      try
      {

        _context = result.AsyncState as CustomerOrderContext;
        _context.EndSaveChanges(result);
        NotifySaveStatusChanged("Saved");

      }
      catch (Exception ex)
      {

        NotifySaveStatusChanged("Error:" + ex.Message);
      }
    }
    

    public event PropertyChangedEventHandler PropertyChanged;
    private void NotifyPropertyChanged(String propertyName)
    {
      PropertyChangedEventHandler handler = PropertyChanged;
      if (null != handler)
      {
        handler(this, new PropertyChangedEventArgs(propertyName));
      }
    }
    public event SaveStatusChangedEventHandler SaveStatusChanged;
    private void NotifySaveStatusChanged(String status)
    {
      SaveStatusChangedEventHandler handler = SaveStatusChanged;
      if (null != handler)
      {
        handler(this, new PropertyChangedEventArgs(status));
      }
    }
   

    #region LineItems

  public void LoadLineItems(int orderId)
    {
     
      this.CurrentOrderNumber = Orders.Where(o => o.OrderId == orderId).FirstOrDefault().OrderNumber;
      var qry = (DataServiceQuery<LineItem>)(from l in _context.LineItems.Expand("Product") where l.OrderId == orderId select l);

      try
      {
        qry.BeginExecute(OnLineItemsLoaded, qry);
      }
      catch (DataServiceQueryException ex)
      {
        throw new ApplicationException(
            "An error occurred during query execution.", ex);
      }

    }

    private void OnLineItemsLoaded(IAsyncResult result)
    {
      try
      {
        var query =
            result.AsyncState as DataServiceQuery<LineItem>;

        var lineItems = query.EndExecute(result).ToList();
        //avoid threading issue by modifying this locally scoped list
        var lineItemVMs = new List<LineItemViewModel>();

        foreach (var item in lineItems)
        {
          lineItemVMs.Add(new LineItemViewModel
          {
            OrderId = item.OrderId,
            LineItemId = item.LineItemId,
            ProductName = item.Product.Name,
            Quantity = item.OrderQty,
            UnitPrice = item.UnitPrice
          });
        }
        
        this.LineItems = new ObservableCollection<LineItemViewModel>(lineItemVMs);
        NotifyPropertyChanged("LineItem");
      }
      catch (Exception ex)
      {

        NotifyPropertyChanged("Error:" + ex.Message);
      }


      this.IsDataLoaded = true;
    }

  

    #endregion
  }
}